@@ -26,8 +26,7 @@
  * needs information about a file.  They are passed a NautilusFileInfo 
  * object which should be filled with relevant information */
 
-#ifndef NAUTILUS_INFO_PROVIDER_H
-#define NAUTILUS_INFO_PROVIDER_H
+#pragma once
 
 #if !defined (NAUTILUS_EXTENSION_H) && !defined (NAUTILUS_COMPILATION)
 #warning "Only <nautilus-extension.h> should be included directly."
@@ -124,6 +123,4 @@ void                    nautilus_info_provider_update_complete_invoke (GClosure
                                                                        NautilusOperationHandle  *handle,
                                                                        NautilusOperationResult   result);
 
-G_END_DECLS
-
-#endif
+G_END_DECLS
\ No newline at end of file
