@@ -42,7 +42,6 @@
 #include "nautilus-global-preferences.h"
 #include "nautilus-canvas-container.h"
 #include "nautilus-canvas-dnd.h"
-#include "nautilus-link.h"
 #include "nautilus-metadata.h"
 #include "nautilus-clipboard.h"
 
@@ -356,12 +355,6 @@ nautilus_canvas_view_add_files (NautilusFilesView *view,
     canvas_view = NAUTILUS_CANVAS_VIEW (view);
     canvas_container = get_canvas_container (canvas_view);
 
-    /* Reset scroll region for the first canvas added when loading a directory. */
-    if (nautilus_files_view_get_loading (view) && nautilus_canvas_container_is_empty (canvas_container))
-    {
-        nautilus_canvas_container_reset_scroll_region (canvas_container);
-    }
-
     for (l = files; l != NULL; l = l->next)
     {
         if (nautilus_canvas_container_add (canvas_container,
@@ -915,7 +908,7 @@ nautilus_canvas_view_select_first (NautilusFilesView *view)
 static void
 nautilus_canvas_view_reveal_selection (NautilusFilesView *view)
 {
-    GList *selection;
+    g_autolist (NautilusFile) selection = NULL;
 
     g_return_if_fail (NAUTILUS_IS_CANVAS_VIEW (view));
 
@@ -930,8 +923,6 @@ nautilus_canvas_view_reveal_selection (NautilusFilesView *view)
             (get_canvas_container (NAUTILUS_CANVAS_VIEW (view)),
             selection->data);
     }
-
-    nautilus_file_list_free (selection);
 }
 
 static GdkRectangle *
@@ -962,7 +953,7 @@ get_rectangle_for_data (NautilusFilesView      *view,
 static GdkRectangle *
 nautilus_canvas_view_compute_rename_popover_pointing_to (NautilusFilesView *view)
 {
-    g_autoptr (GList) selection = NULL;
+    g_autolist (NautilusFile) selection = NULL;
     NautilusCanvasIconData *data;
 
     g_return_val_if_fail (NAUTILUS_IS_CANVAS_VIEW (view), NULL);
@@ -979,7 +970,7 @@ nautilus_canvas_view_compute_rename_popover_pointing_to (NautilusFilesView *view
 static GdkRectangle *
 nautilus_canvas_view_reveal_for_selection_context_menu (NautilusFilesView *view)
 {
-    g_autoptr (GList) selection = NULL;
+    g_autolist (NautilusFile) selection = NULL;
     NautilusCanvasContainer *container;
     NautilusCanvasIconData *data;
 
@@ -1237,7 +1228,7 @@ get_icon_drop_target_uri_callback (NautilusCanvasContainer *container,
     g_return_val_if_fail (NAUTILUS_IS_FILE (file), NULL);
     g_return_val_if_fail (NAUTILUS_IS_CANVAS_VIEW (canvas_view), NULL);
 
-    return nautilus_file_get_target_uri (file);
+    return nautilus_file_get_uri (file);
 }
 
 /* Preferences changed callbacks */
@@ -1433,18 +1424,6 @@ initialize_canvas_container (NautilusCanvasView      *canvas_view,
     gtk_widget_show (GTK_WIDGET (canvas_container));
 }
 
-/* Handles an URL received from Mozilla */
-static void
-canvas_view_handle_netscape_url (NautilusCanvasContainer *container,
-                                 const char              *encoded_url,
-                                 const char              *target_uri,
-                                 GdkDragAction            action,
-                                 NautilusCanvasView      *view)
-{
-    nautilus_files_view_handle_netscape_url_drop (NAUTILUS_FILES_VIEW (view),
-                                                  encoded_url, target_uri, action);
-}
-
 static void
 canvas_view_handle_uri_list (NautilusCanvasContainer *container,
                              const char              *item_uris,
@@ -1634,8 +1613,6 @@ nautilus_canvas_view_init (NautilusCanvasView *canvas_view)
                               G_CALLBACK (text_attribute_names_changed_callback),
                               canvas_view);
 
-    g_signal_connect_object (canvas_container, "handle-netscape-url",
-                             G_CALLBACK (canvas_view_handle_netscape_url), canvas_view, 0);
     g_signal_connect_object (canvas_container, "handle-uri-list",
                              G_CALLBACK (canvas_view_handle_uri_list), canvas_view, 0);
     g_signal_connect_object (canvas_container, "handle-text",
