@@ -30,8 +30,6 @@
 #define DEBUG_FLAG NAUTILUS_DEBUG_DBUS
 #include "nautilus-debug.h"
 
-#include <gio/gio.h>
-
 struct _NautilusDBusManager
 {
     GObject parent;
@@ -39,11 +37,6 @@ struct _NautilusDBusManager
     NautilusDBusFileOperations *file_operations;
 };
 
-struct _NautilusDBusManagerClass
-{
-    GObjectClass parent_class;
-};
-
 G_DEFINE_TYPE (NautilusDBusManager, nautilus_dbus_manager, G_TYPE_OBJECT);
 
 static void
@@ -112,7 +105,7 @@ handle_copy_uris (NautilusDBusFileOperations  *object,
                                        g_file_new_for_uri (sources[idx]));
     }
 
-    nautilus_file_operations_copy (source_files, dest_dir, NULL, NULL, NULL);
+    nautilus_file_operations_copy_async (source_files, dest_dir, NULL, NULL, NULL);
 
     g_list_free_full (source_files, g_object_unref);
     g_object_unref (dest_dir);
@@ -161,8 +154,7 @@ nautilus_dbus_manager_class_init (NautilusDBusManagerClass *klass)
 NautilusDBusManager *
 nautilus_dbus_manager_new (void)
 {
-    return g_object_new (nautilus_dbus_manager_get_type (),
-                         NULL);
+    return g_object_new (NAUTILUS_TYPE_DBUS_MANAGER, NULL);
 }
 
 gboolean
