@@ -19,27 +19,29 @@
  *  Author: Darin Adler <darin@bentspoon.com>
  */
 
-#include <config.h>
 #include "nautilus-directory-private.h"
 
-#include "nautilus-directory-notify.h"
-#include "nautilus-file-attributes.h"
-#include "nautilus-file-private.h"
-#include "nautilus-file-utilities.h"
-#include "nautilus-search-directory.h"
-#include "nautilus-starred-directory.h"
-#include "nautilus-search-directory-file.h"
-#include "nautilus-vfs-file.h"
-#include "nautilus-global-preferences.h"
-#include "nautilus-lib-self-check-functions.h"
-#include "nautilus-metadata.h"
-#include "nautilus-profile.h"
-#include "nautilus-vfs-directory.h"
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#include "nautilus-directory-notify.h"
+#include "nautilus-directory-private.h"
+#include "nautilus-enums.h"
+#include "nautilus-file-private.h"
+#include "nautilus-file-queue.h"
+#include "nautilus-file-utilities.h"
+#include "nautilus-global-preferences.h"
+#include "nautilus-lib-self-check-functions.h"
+#include "nautilus-metadata.h"
+#include "nautilus-profile.h"
+#include "nautilus-search-directory-file.h"
+#include "nautilus-search-directory.h"
+#include "nautilus-starred-directory.h"
+#include "nautilus-vfs-directory.h"
+#include "nautilus-vfs-file.h"
+
 enum
 {
     FILES_ADDED,
@@ -692,6 +694,36 @@ nautilus_directory_new_file_from_filename (NautilusDirectory *directory,
                                                                                               self_owned);
 }
 
+glong
+nautilus_directory_get_max_child_name_length (NautilusDirectory *self)
+{
+    g_autoptr (GFile) location = NULL;
+    g_autofree gchar *path = NULL;
+    glong path_length;
+    glong name_max;
+    glong path_max;
+    glong result;
+
+    g_return_val_if_fail (NAUTILUS_IS_DIRECTORY (self), -1);
+
+    location = nautilus_directory_get_location (self);
+    path = g_file_get_path (location);
+    path_length = strlen (path);
+
+    g_assert (path != NULL);
+
+    name_max = pathconf (path, _PC_NAME_MAX);
+    if (name_max == -1)
+    {
+        return -1;
+    }
+    path_max = pathconf (path, _PC_PATH_MAX);
+    /* Subtracting one from NAME_MAX, since the length excludes the null terminator. */
+    result = MAX (MIN (path_max - path_length, name_max - 1), 0);
+
+    return result;
+}
+
 static GList *
 nautilus_directory_provider_get_all (void)
 {
@@ -699,6 +731,10 @@ nautilus_directory_provider_get_all (void)
     GList *extensions;
 
     extension_point = g_io_extension_point_lookup (NAUTILUS_DIRECTORY_PROVIDER_EXTENSION_POINT_NAME);
+    if (extension_point == NULL)
+    {
+        g_warning ("Directory provider extension point not registered. Did you call nautilus_ensure_extension_points()?");
+    }
     extensions = g_io_extension_point_get_extensions (extension_point);
 
     return extensions;
@@ -1321,7 +1357,6 @@ nautilus_directory_notify_files_changed (GList *files)
              * a changed signal.
              */
             file->details->file_info_is_up_to_date = FALSE;
-            file->details->link_info_is_up_to_date = FALSE;
             nautilus_file_invalidate_extension_info_internal (file);
 
             hash_table_list_prepend (changed_lists, directory, file);
@@ -1338,7 +1373,6 @@ nautilus_directory_notify_files_removed (GList *files)
 {
     GHashTable *changed_lists;
     GList *p;
-    NautilusDirectory *directory;
     GHashTable *parent_directories;
     NautilusFile *file;
     GFile *location;
@@ -1352,6 +1386,8 @@ nautilus_directory_notify_files_removed (GList *files)
     /* Go through all the notifications. */
     for (p = files; p != NULL; p = p->next)
     {
+        NautilusDirectory *directory;
+
         location = p->data;
 
         /* Update file count for parent directory if anyone might care. */
@@ -1366,8 +1402,6 @@ nautilus_directory_notify_files_removed (GList *files)
         file = nautilus_file_get_existing (location);
         if (file != NULL && !nautilus_file_rename_in_progress (file))
         {
-            NautilusDirectory *directory;
-
             directory = nautilus_file_get_directory (file);
 
             /* Mark it gone and prepare to send the changed signal. */
@@ -1926,7 +1960,6 @@ nautilus_directory_list_sort_by_uri (GList *list)
 #if !defined (NAUTILUS_OMIT_SELF_CHECK)
 
 #include <eel/eel-debug.h>
-#include "nautilus-file-attributes.h"
 
 static int data_dummy;
 static gboolean got_files_flag;
