@@ -19,12 +19,12 @@
    Author: Darin Adler <darin@bentspoon.com>
 */
 
-#ifndef NAUTILUS_DIRECTORY_H
-#define NAUTILUS_DIRECTORY_H
+#pragma once
 
 #include <gtk/gtk.h>
 #include <gio/gio.h>
-#include "nautilus-file-attributes.h"
+
+#include "nautilus-enums.h"
 
 /* NautilusDirectory is a class that manages the model for a directory,
    real or virtual, for Nautilus, mainly the file-manager component. The directory is
@@ -58,13 +58,14 @@
 typedef struct NautilusFile NautilusFile;
 #endif
 
-typedef struct NautilusDirectoryDetails NautilusDirectoryDetails;
+typedef struct _NautilusDirectory        NautilusDirectory;
+typedef struct  NautilusDirectoryDetails NautilusDirectoryDetails;
 
-typedef struct
+struct _NautilusDirectory
 {
 	GObject object;
 	NautilusDirectoryDetails *details;
-} NautilusDirectory;
+};
 
 typedef void (*NautilusDirectoryCallback) (NautilusDirectory *directory,
 					   GList             *files,
@@ -252,4 +253,15 @@ NautilusFile *     nautilus_directory_new_file_from_filename   (NautilusDirector
                                                                 const char        *filename,
                                                                 gboolean           self_owned);
 
-#endif /* NAUTILUS_DIRECTORY_H */
+/**
+ * nautilus_directory_get_max_child_name_length:
+ * @directory: a #NautilusDirectory
+ *
+ * Gets the maximum file name length for files inside @directory.
+ *
+ * This call does blocking I/O.
+ *
+ * Returns: The maximum file name length in bytes, -1 if the maximum length
+ *          could not be determined or 0 if @directory path is too long.
+ */
+glong nautilus_directory_get_max_child_name_length (NautilusDirectory *directory);
