@@ -20,8 +20,7 @@
    Authors: Cosimo Cecchi <cosimoc@gnome.org>
 */
 
-#ifndef NAUTILUS_FILE_CONFLICT_DIALOG_H
-#define NAUTILUS_FILE_CONFLICT_DIALOG_H
+#pragma once
 
 #include <glib-object.h>
 #include <gio/gio.h>
@@ -56,6 +55,4 @@ void nautilus_file_conflict_dialog_disable_apply_to_all (NautilusFileConflictDia
 char*      nautilus_file_conflict_dialog_get_new_name     (NautilusFileConflictDialog *dialog);
 gboolean   nautilus_file_conflict_dialog_get_apply_to_all (NautilusFileConflictDialog *dialog);
 
-G_END_DECLS
-
-#endif /* NAUTILUS_FILE_CONFLICT_DIALOG_H */
+G_END_DECLS
\ No newline at end of file
