@@ -17,8 +17,7 @@
  *
  */
 
-#ifndef NAUTILUS_FILE_NAME_WIDGET_CONTROLLER_H
-#define NAUTILUS_FILE_NAME_WIDGET_CONTROLLER_H
+#pragma once
 
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -49,4 +48,3 @@ gchar * nautilus_file_name_widget_controller_get_new_name (NautilusFileNameWidge
 
 void    nautilus_file_name_widget_controller_set_containing_directory (NautilusFileNameWidgetController *controller,
                                                                        NautilusDirectory                *directory);
-#endif /* NAUTILUS_FILE_NAME_WIDGET_CONTROLLER_H */
