@@ -21,8 +21,7 @@
             Pavel Cisler <pavel@eazel.com>
 */
 
-#ifndef NAUTILUS_FILE_OPERATIONS_H
-#define NAUTILUS_FILE_OPERATIONS_H
+#pragma once
 
 #include <gtk/gtk.h>
 #include <gio/gio.h>
@@ -84,14 +83,23 @@ void nautilus_file_operations_new_file_from_template (GtkWidget               *p
 						      NautilusCreateCallback   done_callback,
 						      gpointer                 data);
 
-void nautilus_file_operations_delete          (GList                  *files,
-					       GtkWindow              *parent_window,
-					       NautilusDeleteCallback  done_callback,
-					       gpointer                done_callback_data);
-void nautilus_file_operations_trash_or_delete (GList                  *files,
-					       GtkWindow              *parent_window,
-					       NautilusDeleteCallback  done_callback,
-					       gpointer                done_callback_data);
+void nautilus_file_operations_trash_or_delete_sync (GList                  *files,
+                                                    GtkWindow              *parent_window,
+                                                    NautilusDeleteCallback  done_callback,
+                                                    gpointer                done_callback_data);
+void nautilus_file_operations_delete_sync (GList                  *files,
+                                           GtkWindow              *parent_window,
+                                           NautilusDeleteCallback  done_callback,
+                                           gpointer                done_callback_data);
+void nautilus_file_operations_trash_or_delete_async (GList                  *files,
+                                                     GtkWindow              *parent_window,
+                                                     NautilusDeleteCallback  done_callback,
+                                                     gpointer                done_callback_data);
+void nautilus_file_operations_delete_async (GList                  *files,
+                                            GtkWindow              *parent_window,
+                                            NautilusDeleteCallback  done_callback,
+                                            gpointer                done_callback_data);
+
 
 void nautilus_file_set_permissions_recursive (const char                     *directory,
 					      guint32                         file_permissions,
@@ -119,16 +127,28 @@ void nautilus_file_operations_mount_volume_full (GtkWindow
 						 NautilusMountCallback           mount_callback,
 						 GObject                        *mount_callback_data_object);
 
-void nautilus_file_operations_copy      (GList                *files,
-					 GFile                *target_dir,
-					 GtkWindow            *parent_window,
-					 NautilusCopyCallback  done_callback,
-					 gpointer              done_callback_data);
-void nautilus_file_operations_move      (GList                *files,
-					 GFile                *target_dir,
-					 GtkWindow            *parent_window,
-					 NautilusCopyCallback  done_callback,
-					 gpointer              done_callback_data);
+void nautilus_file_operations_copy_async (GList                *files,
+                                          GFile                *target_dir,
+                                          GtkWindow            *parent_window,
+                                          NautilusCopyCallback  done_callback,
+                                          gpointer              done_callback_data);
+void nautilus_file_operations_copy_sync (GList                *files,
+                                         GFile                *target_dir,
+                                         GtkWindow            *parent_window,
+                                         NautilusCopyCallback  done_callback,
+                                         gpointer              done_callback_data);
+
+void nautilus_file_operations_move_async (GList                *files,
+                                          GFile                *target_dir,
+                                          GtkWindow            *parent_window,
+                                          NautilusCopyCallback  done_callback,
+                                          gpointer              done_callback_data);
+void nautilus_file_operations_move_sync (GList                *files,
+                                         GFile                *target_dir,
+                                         GtkWindow            *parent_window,
+                                         NautilusCopyCallback  done_callback,
+                                         gpointer              done_callback_data);
+
 void nautilus_file_operations_duplicate (GList                *files,
 					 GtkWindow            *parent_window,
 					 NautilusCopyCallback  done_callback,
@@ -138,11 +158,6 @@ void nautilus_file_operations_link      (GList                *files,
 					 GtkWindow            *parent_window,
 					 NautilusCopyCallback  done_callback,
 					 gpointer              done_callback_data);
-void nautilus_file_mark_desktop_file_executable (GFile           *file,
-                                                 GtkWindow        *parent_window,
-                                                 gboolean          interactive,
-                                                 NautilusOpCallback done_callback,
-                                                 gpointer          done_callback_data);
 void nautilus_file_operations_extract_files (GList                   *files,
                                              GFile                   *destination_directory,
                                              GtkWindow               *parent_window,
@@ -155,6 +170,3 @@ void nautilus_file_operations_compress (GList                  *files,
                                         GtkWindow              *parent_window,
                                         NautilusCreateCallback  done_callback,
                                         gpointer                done_callback_data);
-
-
-#endif /* NAUTILUS_FILE_OPERATIONS_H */
