@@ -22,8 +22,7 @@
  *
  */
 
-#ifndef __NAUTILUS_FILE_UNDO_OPERATIONS_H__
-#define __NAUTILUS_FILE_UNDO_OPERATIONS_H__
+#pragma once
 
 #include <gio/gio.h>
 #include <gtk/gtk.h>
@@ -408,7 +407,4 @@ GType nautilus_file_undo_info_compress_get_type (void) G_GNUC_CONST;
 NautilusFileUndoInfo * nautilus_file_undo_info_compress_new (GList        *sources,
                                                              GFile        *output,
                                                              AutoarFormat  format,
-                                                             AutoarFilter  filter);
-
-
-#endif /* __NAUTILUS_FILE_UNDO_OPERATIONS_H__ */
+                                                             AutoarFilter  filter);
\ No newline at end of file
