@@ -30,8 +30,8 @@
 #include "nautilus-file-operations.h"
 #include "nautilus-search-directory.h"
 #include "nautilus-starred-directory.h"
+#include "nautilus-ui-utilities.h"
 #include <eel/eel-glib-extensions.h>
-#include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-vfs-extensions.h>
@@ -44,7 +44,6 @@
 
 #define NAUTILUS_USER_DIRECTORY_NAME "nautilus"
 #define DEFAULT_NAUTILUS_DIRECTORY_MODE (0755)
-#define DEFAULT_DESKTOP_DIRECTORY_MODE (0755)
 
 /* Allowed characters outside alphanumeric for unreserved. */
 #define G_URI_OTHER_UNRESERVED "-._~"
@@ -365,7 +364,7 @@ nautilus_get_user_directory (void)
 
     if (!g_file_test (user_directory, G_FILE_TEST_EXISTS))
     {
-        g_mkdir (user_directory, DEFAULT_NAUTILUS_DIRECTORY_MODE);
+        g_mkdir_with_parents (user_directory, DEFAULT_NAUTILUS_DIRECTORY_MODE);
         /* FIXME bugzilla.gnome.org 41286:
          * How should we handle the case where this mkdir fails?
          * Note that nautilus_application_startup will refuse to launch if this
@@ -391,20 +390,6 @@ nautilus_get_scripts_directory_path (void)
     return g_build_filename (g_get_user_data_dir (), "nautilus", "scripts", NULL);
 }
 
-static const char *
-get_desktop_path (void)
-{
-    const char *desktop_path;
-
-    desktop_path = g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP);
-    if (desktop_path == NULL)
-    {
-        desktop_path = g_get_home_dir ();
-    }
-
-    return desktop_path;
-}
-
 char *
 nautilus_get_home_directory_uri (void)
 {
@@ -440,26 +425,6 @@ nautilus_get_templates_directory_uri (void)
     return uri;
 }
 
-/* These need to be reset to NULL when desktop_is_home_dir changes */
-static GFile *desktop_dir = NULL;
-static GFile *desktop_dir_dir = NULL;
-static char *desktop_dir_filename = NULL;
-
-static void
-update_desktop_dir (void)
-{
-    const char *path;
-    char *dirname;
-
-    path = get_desktop_path ();
-    desktop_dir = g_file_new_for_path (path);
-
-    dirname = g_path_get_dirname (path);
-    desktop_dir_dir = g_file_new_for_path (dirname);
-    g_free (dirname);
-    desktop_dir_filename = g_path_get_basename (path);
-}
-
 gboolean
 nautilus_is_home_directory_file (GFile      *dir,
                                  const char *filename)
@@ -506,31 +471,6 @@ nautilus_is_root_directory (GFile *dir)
     return g_file_equal (dir, root_dir);
 }
 
-
-gboolean
-nautilus_is_desktop_directory_file (GFile      *dir,
-                                    const char *file)
-{
-    if (desktop_dir == NULL)
-    {
-        update_desktop_dir ();
-    }
-
-    return (g_file_equal (dir, desktop_dir_dir) &&
-            strcmp (file, desktop_dir_filename) == 0);
-}
-
-gboolean
-nautilus_is_desktop_directory (GFile *dir)
-{
-    if (desktop_dir == NULL)
-    {
-        update_desktop_dir ();
-    }
-
-    return g_file_equal (dir, desktop_dir);
-}
-
 gboolean
 nautilus_is_search_directory (GFile *dir)
 {
@@ -586,7 +526,7 @@ nautilus_is_other_locations_directory (GFile *dir)
 GMount *
 nautilus_get_mounted_mount_for_root (GFile *location)
 {
-    GVolumeMonitor *volume_monitor;
+    g_autoptr (GVolumeMonitor) volume_monitor = NULL;
     GList *mounts;
     GList *l;
     GMount *mount;
@@ -762,40 +702,6 @@ nautilus_is_file_roller_installed (void)
     return installed > 0 ? TRUE : FALSE;
 }
 
-/* Returns TRUE if the file is in XDG_DATA_DIRS. This is used for
- *  deciding if a desktop file is "trusted" based on the path */
-gboolean
-nautilus_is_in_system_dir (GFile *file)
-{
-    const char * const *data_dirs;
-    char *path;
-    int i;
-    gboolean res;
-
-    if (!g_file_is_native (file))
-    {
-        return FALSE;
-    }
-
-    path = g_file_get_path (file);
-
-    res = FALSE;
-
-    data_dirs = g_get_system_data_dirs ();
-    for (i = 0; path != NULL && data_dirs[i] != NULL; i++)
-    {
-        if (g_str_has_prefix (path, data_dirs[i]))
-        {
-            res = TRUE;
-            break;
-        }
-    }
-
-    g_free (path);
-
-    return res;
-}
-
 GHashTable *
 nautilus_trashed_files_get_original_directories (GList  *files,
                                                  GList **unhandled_files)
@@ -911,7 +817,7 @@ ensure_dirs_task_ready_cb (GObject      *_source,
         files = g_hash_table_lookup (data->original_dirs_hash, original_dir);
         locations = locations_from_file_list (files);
 
-        nautilus_file_operations_move
+        nautilus_file_operations_move_sync
             (locations,
             original_dir_location,
             data->parent_window,
@@ -986,9 +892,10 @@ nautilus_restore_files_from_trash (GList     *files,
         message = g_strdup_printf (_("Could not determine original location of “%s” "), file_name);
         g_free (file_name);
 
-        eel_show_warning_dialog (message,
-                                 _("The item cannot be restored from trash"),
-                                 parent_window);
+        show_dialog (message,
+                     _("The item cannot be restored from trash"),
+                     parent_window,
+                     GTK_MESSAGE_WARNING);
         g_free (message);
     }
 
@@ -1484,3 +1391,52 @@ nautilus_uri_to_native_uri (const gchar *uri)
 
     return NULL;
 }
+
+NautilusQueryRecursive
+location_settings_search_get_recursive (void)
+{
+    switch (g_settings_get_enum (nautilus_preferences, "recursive-search"))
+    {
+        case NAUTILUS_SPEED_TRADEOFF_ALWAYS:
+        {
+            return NAUTILUS_QUERY_RECURSIVE_ALWAYS;
+        }
+        break;
+        
+        case NAUTILUS_SPEED_TRADEOFF_LOCAL_ONLY:
+        {
+            return NAUTILUS_QUERY_RECURSIVE_LOCAL_ONLY;
+        }
+        break;
+        
+        case NAUTILUS_SPEED_TRADEOFF_NEVER:
+        {
+             return NAUTILUS_QUERY_RECURSIVE_NEVER;
+        }
+        break;
+    }
+
+    return NAUTILUS_QUERY_RECURSIVE_ALWAYS;
+}
+
+NautilusQueryRecursive
+location_settings_search_get_recursive_for_location (GFile *location)
+{
+    NautilusQueryRecursive recursive = location_settings_search_get_recursive ();
+
+    g_return_val_if_fail (location, recursive);
+
+    if (recursive == NAUTILUS_QUERY_RECURSIVE_LOCAL_ONLY)
+    {
+        g_autoptr (NautilusFile) file = nautilus_file_get_existing (location);
+
+        g_return_val_if_fail (file != NULL, recursive);
+
+        if (nautilus_file_is_remote (file))
+        {
+            recursive = NAUTILUS_QUERY_RECURSIVE_NEVER;
+        }
+    }
+
+    return recursive;
+}
