@@ -16,8 +16,7 @@
  * 
  */
 
-#ifndef NAUTILUS_PATHBAR_H
-#define NAUTILUS_PATHBAR_H
+#pragma once
 
 #include <gtk/gtk.h>
 #include <gio/gio.h>
@@ -36,5 +35,7 @@ struct _NautilusPathBarClass
                                      GtkPlacesOpenFlags flags);
 };
 void     nautilus_path_bar_set_path    (NautilusPathBar *self, GFile *file);
-
-#endif /* NAUTILUS_PATHBAR_H */
+void     nautilus_path_bar_set_extensions_background_menu (NautilusPathBar *self,
+                                                           GMenu           *menu);
+void     nautilus_path_bar_set_templates_menu (NautilusPathBar *self,
+                                               GMenu           *menu);
