@@ -18,11 +18,15 @@
 
 #include "nautilus-mime-actions.h"
 #include "nautilus-places-view.h"
-#include "nautilus-window-slot.h"
-#include "nautilus-application.h"
-#include "nautilus-toolbar-menu-sections.h"
+
 #include "gtk/nautilusgtkplacesviewprivate.h"
 
+#include "nautilus-application.h"
+#include "nautilus-file.h"
+#include "nautilus-toolbar-menu-sections.h"
+#include "nautilus-view.h"
+#include "nautilus-window-slot.h"
+
 typedef struct
 {
     GFile *location;
@@ -48,8 +52,9 @@ enum
     PROP_0,
     PROP_LOCATION,
     PROP_SEARCH_QUERY,
-    PROP_IS_LOADING,
-    PROP_IS_SEARCHING,
+    PROP_LOADING,
+    PROP_SEARCHING,
+    PROP_SELECTION,
     LAST_PROP
 };
 
@@ -108,7 +113,7 @@ open_location_cb (NautilusPlacesView *view,
 static void
 loading_cb (NautilusView *view)
 {
-    g_object_notify (G_OBJECT (view), "is-loading");
+    g_object_notify (G_OBJECT (view), "loading");
 }
 
 static void
@@ -345,8 +350,8 @@ nautilus_places_view_class_init (NautilusPlacesViewClass *klass)
     object_class->get_property = nautilus_places_view_get_property;
     object_class->set_property = nautilus_places_view_set_property;
 
-    g_object_class_override_property (object_class, PROP_IS_LOADING, "is-loading");
-    g_object_class_override_property (object_class, PROP_IS_SEARCHING, "is-searching");
+    g_object_class_override_property (object_class, PROP_LOADING, "loading");
+    g_object_class_override_property (object_class, PROP_SEARCHING, "searching");
     g_object_class_override_property (object_class, PROP_LOCATION, "location");
     g_object_class_override_property (object_class, PROP_SEARCH_QUERY, "search-query");
 }
