@@ -51,6 +51,8 @@
 /* bool preferences */
 #define NAUTILUS_PREFERENCES_DIALOG_FOLDERS_FIRST_WIDGET                       \
     "sort_folders_first_checkbutton"
+#define NAUTILUS_PREFERENCS_DIALOG_START_WITH_SIDEBAR                          \
+    "show_sidebar_checkbutton"
 #define NAUTILUS_PREFERENCES_DIALOG_DELETE_PERMANENTLY_WIDGET                  \
     "show_delete_permanently_checkbutton"
 #define NAUTILUS_PREFERENCES_DIALOG_CREATE_LINK_WIDGET                         \
@@ -59,7 +61,7 @@
     "use_tree_view_checkbutton"
 #define NAUTILUS_PREFERENCES_DIALOG_TRASH_CONFIRM_WIDGET                       \
     "trash_confirm_checkbutton"
-#define NAUTILUS_PREFERENCES_DIALOG_USE_NEW_VIEWS_WIDGET                    \
+#define NAUTILUS_PREFERENCES_DIALOG_USE_NEW_VIEWS_WIDGET                       \
     "use_new_views_checkbutton"
 
 /* int enums */
@@ -354,16 +356,15 @@ nautilus_preferences_window_setup_list_column_page (GtkBuilder *builder)
     gtk_box_pack_start (GTK_BOX (box), chooser, TRUE, TRUE, 0);
 }
 
-static gboolean format_spin_button(GtkSpinButton *spin_button,
-                                   gpointer       user_data)
+static gboolean
+format_spin_button (GtkSpinButton *spin_button,
+                    gpointer       user_data)
 {
-    GtkAdjustment *adjustment;
-    int value;
+    gint value;
     gchar *text;
 
-    adjustment = gtk_spin_button_get_adjustment (spin_button);
-    value = (int) gtk_adjustment_get_value (adjustment);
-    text = g_strdup_printf ("%d MB", value);
+    value = gtk_spin_button_get_value_as_int (spin_button);
+    text = g_strdup_printf (_("%d MB"), value);
     gtk_entry_set_text (GTK_ENTRY (spin_button), text);
 
     return TRUE;
@@ -461,6 +462,9 @@ static void nautilus_preferences_window_setup(GtkBuilder *builder,
     bind_builder_bool (builder, gtk_filechooser_preferences,
                        NAUTILUS_PREFERENCES_DIALOG_FOLDERS_FIRST_WIDGET,
                        NAUTILUS_PREFERENCES_SORT_DIRECTORIES_FIRST);
+    bind_builder_bool (builder, nautilus_window_state,
+                       NAUTILUS_PREFERENCS_DIALOG_START_WITH_SIDEBAR,
+                       NAUTILUS_WINDOW_STATE_START_WITH_SIDEBAR);
     bind_builder_bool (builder, nautilus_preferences,
                        NAUTILUS_PREFERENCES_DIALOG_TRASH_CONFIRM_WIDGET,
                        NAUTILUS_PREFERENCES_CONFIRM_TRASH);
@@ -473,9 +477,6 @@ static void nautilus_preferences_window_setup(GtkBuilder *builder,
     bind_builder_bool (builder, nautilus_preferences,
                        NAUTILUS_PREFERENCES_DIALOG_DELETE_PERMANENTLY_WIDGET,
                        NAUTILUS_PREFERENCES_SHOW_DELETE_PERMANENTLY);
-    bind_builder_bool (builder, nautilus_preferences,
-                       NAUTILUS_PREFERENCES_DIALOG_USE_NEW_VIEWS_WIDGET,
-                       NAUTILUS_PREFERENCES_USE_EXPERIMENTAL_VIEWS);
 
     bind_builder_radio (
         builder, nautilus_preferences, (const char **) click_behavior_components,
