@@ -19,14 +19,12 @@
  *
  */
 
-#ifndef NAUTILUS_SEARCH_ENGINE_H
-#define NAUTILUS_SEARCH_ENGINE_H
+#pragma once
 
 #include <glib-object.h>
 
 #include "nautilus-directory.h"
 #include "nautilus-search-engine-model.h"
-#include "nautilus-search-engine-simple.h"
 
 G_BEGIN_DECLS
 
@@ -42,9 +40,5 @@ struct _NautilusSearchEngineClass
 NautilusSearchEngine *nautilus_search_engine_new                (void);
 NautilusSearchEngineModel *
                       nautilus_search_engine_get_model_provider (NautilusSearchEngine *engine);
-NautilusSearchEngineSimple *
-                      nautilus_search_engine_get_simple_provider (NautilusSearchEngine *engine);
 
 G_END_DECLS
-
-#endif /* NAUTILUS_SEARCH_ENGINE_H */
