@@ -19,13 +19,14 @@
    Author: Christian Neumair <cneumair@gnome.org>
 */
 
-#ifndef NAUTILUS_WINDOW_SLOT_H
-#define NAUTILUS_WINDOW_SLOT_H
+#pragma once
 
 #include <gdk/gdk.h>
 #include <gio/gio.h>
 #include <gtk/gtk.h>
 
+#include "nautilus-types.h"
+
 typedef enum {
 	NAUTILUS_LOCATION_CHANGE_STANDARD,
 	NAUTILUS_LOCATION_CHANGE_BACK,
@@ -36,12 +37,6 @@ typedef enum {
 #define NAUTILUS_TYPE_WINDOW_SLOT (nautilus_window_slot_get_type ())
 G_DECLARE_DERIVABLE_TYPE (NautilusWindowSlot, nautilus_window_slot, NAUTILUS, WINDOW_SLOT, GtkBox)
 
-#include "nautilus-query-editor.h"
-#include "nautilus-files-view.h"
-#include "nautilus-view.h"
-#include "nautilus-window.h"
-#include "nautilus-toolbar-menu-sections.h"
-
 typedef struct
 {
     NautilusFile *file;
@@ -97,7 +92,7 @@ const gchar *nautilus_window_slot_get_title                (NautilusWindowSlot *
 void         nautilus_window_slot_update_title		   (NautilusWindowSlot *slot);
 
 gboolean nautilus_window_slot_handle_event       	   (NautilusWindowSlot *slot,
-							    GdkEventKey        *event);
+							    GdkEvent           *event);
 
 void    nautilus_window_slot_queue_reload		   (NautilusWindowSlot *slot);
 
@@ -105,14 +100,22 @@ GIcon*   nautilus_window_slot_get_icon                     (NautilusWindowSlot *
 
 NautilusToolbarMenuSections * nautilus_window_slot_get_toolbar_menu_sections (NautilusWindowSlot *slot);
 
+GMenu* nautilus_window_slot_get_templates_menu (NautilusWindowSlot *self);
+
+GMenu* nautilus_window_slot_get_extensions_background_menu (NautilusWindowSlot *self);
+
 gboolean nautilus_window_slot_get_active                   (NautilusWindowSlot *slot);
 
 void     nautilus_window_slot_set_active                   (NautilusWindowSlot *slot,
                                                             gboolean            active);
 gboolean nautilus_window_slot_get_loading                  (NautilusWindowSlot *slot);
 
+gboolean nautilus_window_slot_get_searching                (NautilusWindowSlot *slot);
+
+GList* nautilus_window_slot_get_selection                  (NautilusWindowSlot *slot);
+
 void     nautilus_window_slot_search                       (NautilusWindowSlot *slot,
-                                                            const gchar        *text);
+                                                            NautilusQuery      *query);
 
 gboolean nautilus_window_slot_handles_location (NautilusWindowSlot *self,
                                                 GFile              *location);
@@ -122,7 +125,7 @@ void nautilus_window_slot_restore_from_data (NautilusWindowSlot *self,
 
 RestoreTabData* nautilus_window_slot_get_restore_tab_data (NautilusWindowSlot *self);
 
+NautilusQueryEditor *nautilus_window_slot_get_query_editor (NautilusWindowSlot *self);
+
 /* Only used by slot-dnd */
 NautilusView*  nautilus_window_slot_get_current_view       (NautilusWindowSlot *slot);
-
-#endif /* NAUTILUS_WINDOW_SLOT_H */
